using System;
using System.Collections.Generic;
using Ato.EN.IntegrationServices.Document.DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationTPARPYE;


namespace Ato.EN.IntegrationServices.CodeGenerationTPAR {
    public partial class TPAR2021Validator
    {
        /// <summary>
        /// The Maximum Severity of the process messages returned from the ValidateReport function.
        /// </summary>
        public ProcessMessageSeverity MaximumSeverity = ProcessMessageSeverity.Progressive;

        /// <summary>
        /// The Maximum Severity of the process messages returned from the ValidateCrossformReport function, in relation to the parent.
        /// </summary>
        public ProcessMessageSeverity MaximumSeverityCrossFormParent = ProcessMessageSeverity.Progressive;

        /// <summary>
        /// The Maximum Severity of the process messages returned from the ValidateCrossformReport function, in relation to the child.
        /// </summary>
        public ProcessMessageSeverity MaximumSeverityCrossFormChild = ProcessMessageSeverity.Progressive;





        #region VR.ATO.TPAR.000062
        /*  VR.ATO.TPAR.000062
                  At least one TPARPYE child must be provided.

              Legacy Rule Format:
                  Count(^TPARPYE) = 0

          Technical Business Rule Format:
                  Count(^TPARPYE) = 0

      Data Elements:

          ^TPARPYE = TPARPYE
          */
        protected void VRATOTPAR000062(List<ProcessMessageDocument> response, TPAR2021 report, int childCount)
        {
            ProcessMessageDocument processMessage;
            bool assertion;

            assertion = (childCount < 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.TPAR.000062",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"At least one TPARPYE child must be provided",
                    //Location = "/tns:TPAR/",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TPAR.000062" } },
                };
                response.Add(processMessage);
              
            }
        }

    #endregion // VR.ATO.TPAR.000062

        #region VR.ATO.TPARPYE.W00036

        /*  VR.ATO.TPARPYE.W00036
                    Date of payment must be the same year as the financial year the payment was made

                Legacy Rule Format:
                    (FinancialYear(^TPARPYE135) <> ^TPAR47)

            Technical Business Rule Format:
                    (FinancialYear(^TPARPYE135) <> ^TPAR47)

        Data Elements:
    
            ^TPARPYE135 = TPARPYE:Payee:GovernmentFunding.GrantPayment.Date
    
            ^TPAR47 = TPAR47
            */
        protected void VRATOTPARPYEW00036(List<ProcessMessageDocument> response, TPAR2021 report, TPARPYE2021 childReport)
        {

            ProcessMessageDocument processMessage;
            bool assertion;

            //TODO: Write manual rule for VR.ATO.TPARPYE.W00036

            if ((childReport != null) && (report.TPAR47 != null) && (childReport.TPARPYE135 != null))
                {

                assertion = (FinancialYear(childReport.TPARPYE135.GetValueOrDefault()) != report.TPAR47.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.TPARPYE.W00036",
                        Severity = ProcessMessageSeverity.Warning,
                        Description = @"Date of payment must be the same year as the financial year the payment was made",
                        Location = "/tns:TPARPYE/tns:Payee/tns:GovernmentFundingGrantPaymentD",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.TPARPYE.W00036" } },
                    };

                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "TPARPYE135", Value = GetValueOrEmptyDate(childReport.TPARPYE135) });

                    response.Add(processMessage);
                }
            }
        }

        #endregion // VR.ATO.TPARPYE.W00036


        #region ValidateCrossFormRules

        public  List<ProcessMessageDocument> ValidateCrossformReport(TPAR2021 report, TPARPYE2021 childReport, int childCount, HashSet<string> failedDocumentIDs, bool generateWarnings = true, string ParentDocumentPartIdentifier = "", string ChildDocumentPartIdentifier = "", bool isLastOrAll = false)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            #region Cross Form Rules
            VRATOTPAR000062(response, report, childCount);
            VRATOTPARPYEW00036(response, report, childReport);
            #endregion Cross Form Rules
            return response;
        }

        #endregion
    }
}
